import json


############################
# Exceptions
############################

# TODO: color and row exceptions control

class BinanceAPIException(Exception):
    """
    Exceptions from https://github.com/sammchardy/python-binance
    """

    def __init__(self, response, status_code, text):
        self.code = 0
        try:
            json_res = json.loads(text)
        except ValueError:
            self.message = 'Invalid JSON error message from Binance: {}'.format(response.text)
        else:
            self.code = json_res['code']
            self.message = json_res['msg']
        self.status_code = status_code
        self.response = response
        self.request = getattr(response, 'request', None)

    def __str__(self):  # pragma: no cover
        return 'APIError(code=%s): %s' % (self.code, self.message)


class BinanceRequestException(Exception):
    """
    Exceptions from https://github.com/sammchardy/python-binance
    """

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return f'BinanceRequestException: {self.message}'


class BinanceOrderException(Exception):
    """
    Exceptions from https://github.com/sammchardy/python-binance
    """
    def __init__(self, code, message):
        self.code = code
        self.message = message

    def __str__(self):
        return 'BinanceOrderException(code=%s): %s' % (self.code, self.message)


class BinanceOrderMinAmountException(BinanceOrderException):
    """
    Exceptions from https://github.com/sammchardy/python-binance
    """
    def __init__(self, value):
        message = "Amount must be a multiple of %s" % value
        super().__init__(-1013, message)


class BinanceOrderMinPriceException(BinanceOrderException):
    """
    Exceptions from https://github.com/sammchardy/python-binance
    """
    def __init__(self, value):
        message = "Price must be at least %s" % value
        super().__init__(-1013, message)


class BinanceOrderMinTotalException(BinanceOrderException):
    """
    Exceptions from https://github.com/sammchardy/python-binance
    """
    def __init__(self, value):
        message = "Total must be at least %s" % value
        super().__init__(-1013, message)


class BinanceOrderUnknownSymbolException(BinanceOrderException):
    """
    Exceptions from https://github.com/sammchardy/python-binance
    """
    def __init__(self, value):
        message = "Unknown symbol %s" % value
        super().__init__(-1013, message)


class BinanceOrderInactiveSymbolException(BinanceOrderException):
    """
    Exceptions from https://github.com/sammchardy/python-binance
    """
    def __init__(self, value):
        message = "Attempting to trade an inactive symbol %s" % value
        super().__init__(-1013, message)


class NotImplementedException(Exception):
    """
    Exceptions from https://github.com/sammchardy/python-binance
    """
    def __init__(self, value):
        message = f'Not implemented: {value}'
        super().__init__(message)
